from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .contact import Contact


@JsonMap({"id_": "id"})
class CardIssuing(BaseModel):
    """CardIssuing

    :param activated_at: Time that the card was activated, in Unix time. Response only., defaults to None
    :type activated_at: float, optional
    :param assigned_at: Time that the card was assigned to a cardholder, in Unix time. Response only., defaults to None
    :type assigned_at: float, optional
    :param blocked_reason: Reason for blocking the card., defaults to None
    :type blocked_reason: str, optional
    :param card_id: ID of the card. String starting with **card_**., defaults to None
    :type card_id: str, optional
    :param card_program: ID of the card program that the card is issued from. String starting with **cardprog_**., defaults to None
    :type card_program: str, optional
    :param country_iso_alpha_2: The country where the card is issued. Two-letter ISO 3166-1 ALPHA-2 code. Response only., defaults to None
    :type country_iso_alpha_2: str, optional
    :param created_at: Time of creation of the issued card object, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param ewallet_contact: ewallet_contact, defaults to None
    :type ewallet_contact: Contact, optional
    :param id_: ID of the Issued Card object. String starting with **ci_**., defaults to None
    :type id_: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param public_details: Details of the issued card., defaults to None
    :type public_details: dict, optional
    :param status: Status of the card, defaults to None
    :type status: str, optional
    :param card_tracking_id: Reserved, defaults to None
    :type card_tracking_id: str, optional
    """

    def __init__(
        self,
        activated_at: float = None,
        assigned_at: float = None,
        blocked_reason: str = None,
        card_id: str = None,
        card_program: str = None,
        country_iso_alpha_2: str = None,
        created_at: float = None,
        ewallet_contact: Contact = None,
        id_: str = None,
        metadata: dict = None,
        public_details: dict = None,
        status: str = None,
        card_tracking_id: str = None,
    ):
        """CardIssuing

        :param activated_at: Time that the card was activated, in Unix time. Response only., defaults to None
        :type activated_at: float, optional
        :param assigned_at: Time that the card was assigned to a cardholder, in Unix time. Response only., defaults to None
        :type assigned_at: float, optional
        :param blocked_reason: Reason for blocking the card., defaults to None
        :type blocked_reason: str, optional
        :param card_id: ID of the card. String starting with **card_**., defaults to None
        :type card_id: str, optional
        :param card_program: ID of the card program that the card is issued from. String starting with **cardprog_**., defaults to None
        :type card_program: str, optional
        :param country_iso_alpha_2: The country where the card is issued. Two-letter ISO 3166-1 ALPHA-2 code. Response only., defaults to None
        :type country_iso_alpha_2: str, optional
        :param created_at: Time of creation of the issued card object, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param ewallet_contact: ewallet_contact, defaults to None
        :type ewallet_contact: Contact, optional
        :param id_: ID of the Issued Card object. String starting with **ci_**., defaults to None
        :type id_: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param public_details: Details of the issued card., defaults to None
        :type public_details: dict, optional
        :param status: Status of the card, defaults to None
        :type status: str, optional
        :param card_tracking_id: Reserved, defaults to None
        :type card_tracking_id: str, optional
        """
        self.activated_at = self._define_number(
            "activated_at", activated_at, nullable=True
        )
        self.assigned_at = self._define_number(
            "assigned_at", assigned_at, nullable=True
        )
        self.blocked_reason = self._define_str(
            "blocked_reason", blocked_reason, nullable=True
        )
        self.card_id = self._define_str("card_id", card_id, nullable=True)
        self.card_program = self._define_str(
            "card_program", card_program, nullable=True
        )
        self.country_iso_alpha_2 = self._define_str(
            "country_iso_alpha_2", country_iso_alpha_2, nullable=True
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.ewallet_contact = self._define_object(ewallet_contact, Contact)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.metadata = metadata
        self.public_details = public_details
        self.status = self._define_str("status", status, nullable=True)
        self.card_tracking_id = self._define_str(
            "card_tracking_id", card_tracking_id, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PaymentIdCaptureBody(BaseModel):
    """PaymentIdCaptureBody

    :param amount: The amount to capture, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
    :type amount: float, optional
    :param receipt_email: Email address that the receipt for this transaction is sent to., defaults to None
    :type receipt_email: str, optional
    :param statement_descriptor: A text description suitable for a customer's payment statement. Some payment methods truncate this string to a limited number of characters., defaults to None
    :type statement_descriptor: str, optional
    """

    def __init__(
        self,
        amount: float = None,
        receipt_email: str = None,
        statement_descriptor: str = None,
    ):
        """PaymentIdCaptureBody

        :param amount: The amount to capture, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
        :type amount: float, optional
        :param receipt_email: Email address that the receipt for this transaction is sent to., defaults to None
        :type receipt_email: str, optional
        :param statement_descriptor: A text description suitable for a customer's payment statement. Some payment methods truncate this string to a limited number of characters., defaults to None
        :type statement_descriptor: str, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.receipt_email = self._define_str(
            "receipt_email", receipt_email, nullable=True
        )
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )



from enum import Enum


class UpdateEwalletStatusStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACT: "ACT"
    :vartype ACT: str
    :cvar DIS: "DIS"
    :vartype DIS: str
    :cvar CLO: "CLO"
    :vartype CLO: str
    """

    ACT = "ACT"
    DIS = "DIS"
    CLO = "CLO"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, UpdateEwalletStatusStatus._member_map_.values())
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap(
    {
        "domain_name": "domainName",
        "epoch_timestamp": "epochTimestamp",
        "expires_at": "expiresAt",
        "merchant_identifier": "merchantIdentifier",
        "merchant_session_identifier": "merchantSessionIdentifier",
    }
)
class ApplePayObjectResponse(BaseModel):
    """ApplePayObjectResponse

    :param display_name: The canonical name for the client's business, suitable for display. 64 or fewer UTF-8 characters. Relevant to Apple Pay., defaults to None
    :type display_name: str, optional
    :param domain_name: The domain name of the client's ecommerce site., defaults to None
    :type domain_name: str, optional
    :param epoch_timestamp: Time of creation of the Apple Pay session, in Unix time., defaults to None
    :type epoch_timestamp: float, optional
    :param expires_at: Time of expiration of the Apple Pay session, in Unix time., defaults to None
    :type expires_at: float, optional
    :param merchant_identifier: Your Apple Pay merchant ID., defaults to None
    :type merchant_identifier: str, optional
    :param merchant_session_identifier: Your Apple Pay session ID., defaults to None
    :type merchant_session_identifier: str, optional
    :param nonce: A string that uniquely identifies each call to Apple Pay., defaults to None
    :type nonce: str, optional
    :param retries: The number of previous attempts to get an Apple Pay Session object., defaults to None
    :type retries: float, optional
    :param signature: Digital signature that validates the authenticity and integrity of a digital wallet payment., defaults to None
    :type signature: str, optional
    """

    def __init__(
        self,
        display_name: str = None,
        domain_name: str = None,
        epoch_timestamp: float = None,
        expires_at: float = None,
        merchant_identifier: str = None,
        merchant_session_identifier: str = None,
        nonce: str = None,
        retries: float = None,
        signature: str = None,
    ):
        """ApplePayObjectResponse

        :param display_name: The canonical name for the client's business, suitable for display. 64 or fewer UTF-8 characters. Relevant to Apple Pay., defaults to None
        :type display_name: str, optional
        :param domain_name: The domain name of the client's ecommerce site., defaults to None
        :type domain_name: str, optional
        :param epoch_timestamp: Time of creation of the Apple Pay session, in Unix time., defaults to None
        :type epoch_timestamp: float, optional
        :param expires_at: Time of expiration of the Apple Pay session, in Unix time., defaults to None
        :type expires_at: float, optional
        :param merchant_identifier: Your Apple Pay merchant ID., defaults to None
        :type merchant_identifier: str, optional
        :param merchant_session_identifier: Your Apple Pay session ID., defaults to None
        :type merchant_session_identifier: str, optional
        :param nonce: A string that uniquely identifies each call to Apple Pay., defaults to None
        :type nonce: str, optional
        :param retries: The number of previous attempts to get an Apple Pay Session object., defaults to None
        :type retries: float, optional
        :param signature: Digital signature that validates the authenticity and integrity of a digital wallet payment., defaults to None
        :type signature: str, optional
        """
        self.display_name = self._define_str(
            "display_name", display_name, nullable=True
        )
        self.domain_name = self._define_str("domain_name", domain_name, nullable=True)
        self.epoch_timestamp = self._define_number(
            "epoch_timestamp", epoch_timestamp, nullable=True
        )
        self.expires_at = self._define_number("expires_at", expires_at, nullable=True)
        self.merchant_identifier = self._define_str(
            "merchant_identifier", merchant_identifier, nullable=True
        )
        self.merchant_session_identifier = self._define_str(
            "merchant_session_identifier", merchant_session_identifier, nullable=True
        )
        self.nonce = self._define_str("nonce", nonce, nullable=True)
        self.retries = self._define_number("retries", retries, nullable=True)
        self.signature = self._define_str("signature", signature, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class FxFee(BaseModel):
    """Describes the fees for processing the currency exchange. Relevant to payouts with FX.

    :param calc_type: Specifies how the fee is calculated. One of the following:  * net - The fee is deducted from the amount paid.  For example, in a payout of $100 with a fee of 5%, the recipient receives $95.00 and the $5.00 fee goes to the client wallet.  * gross - The fee is charged on top of the amount paid.  For example, for a transaction of $100 with a fee of 5%, the sender pays $105.00. The recipient receives $100.00 and the $5.00 fee goes to the client wallet. , defaults to None
    :type calc_type: str, optional
    :param value: The amount of the fee. Percentage., defaults to None
    :type value: float, optional
    """

    def __init__(self, calc_type: str = None, value: float = None):
        """Describes the fees for processing the currency exchange. Relevant to payouts with FX.

        :param calc_type: Specifies how the fee is calculated. One of the following:  * net - The fee is deducted from the amount paid.  For example, in a payout of $100 with a fee of 5%, the recipient receives $95.00 and the $5.00 fee goes to the client wallet.  * gross - The fee is charged on top of the amount paid.  For example, for a transaction of $100 with a fee of 5%, the sender pays $105.00. The recipient receives $100.00 and the $5.00 fee goes to the client wallet. , defaults to None
        :type calc_type: str, optional
        :param value: The amount of the fee. Percentage., defaults to None
        :type value: float, optional
        """
        self.calc_type = self._define_str("calc_type", calc_type, nullable=True)
        self.value = self._define_number("value", value, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class ListCurrenciesResponse(BaseModel):
    """ListCurrenciesResponse

    :param code: Three-letter ISO 4217 alphabetic code for the currency., defaults to None
    :type code: str, optional
    :param digits_after_decimal_separator: Number of digits for the minor currency unit., defaults to None
    :type digits_after_decimal_separator: str, optional
    :param name: Name of the currency in English., defaults to None
    :type name: str, optional
    :param numeric_code: ISO 4217 numeric code for the currency., defaults to None
    :type numeric_code: str, optional
    :param symbol: Unicode symbol for the currency. If there is no official Unicode symbol, this field contains the string **undefined**., defaults to None
    :type symbol: str, optional
    """

    def __init__(
        self,
        code: str = None,
        digits_after_decimal_separator: str = None,
        name: str = None,
        numeric_code: str = None,
        symbol: str = None,
    ):
        """ListCurrenciesResponse

        :param code: Three-letter ISO 4217 alphabetic code for the currency., defaults to None
        :type code: str, optional
        :param digits_after_decimal_separator: Number of digits for the minor currency unit., defaults to None
        :type digits_after_decimal_separator: str, optional
        :param name: Name of the currency in English., defaults to None
        :type name: str, optional
        :param numeric_code: ISO 4217 numeric code for the currency., defaults to None
        :type numeric_code: str, optional
        :param symbol: Unicode symbol for the currency. If there is no official Unicode symbol, this field contains the string **undefined**., defaults to None
        :type symbol: str, optional
        """
        self.code = self._define_str("code", code, nullable=True)
        self.digits_after_decimal_separator = self._define_str(
            "digits_after_decimal_separator",
            digits_after_decimal_separator,
            nullable=True,
        )
        self.name = self._define_str("name", name, nullable=True)
        self.numeric_code = self._define_str(
            "numeric_code", numeric_code, nullable=True
        )
        self.symbol = self._define_str("symbol", symbol, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class HostedPageAdditionalResponseCartItems(BaseModel):
    """Describes the cart items that the customer is purchasing. These items are displayed at the checkout page.

    :param amount: The price of the item in the currency defined in currency., defaults to None
    :type amount: float, optional
    :param name: The name of the item., defaults to None
    :type name: str, optional
    :param quantity: The quantity of the item., defaults to None
    :type quantity: float, optional
    :param image: The image that appears in the checkout page for this item., defaults to None
    :type image: str, optional
    """

    def __init__(
        self,
        amount: float = None,
        name: str = None,
        quantity: float = None,
        image: str = None,
    ):
        """Describes the cart items that the customer is purchasing. These items are displayed at the checkout page.

        :param amount: The price of the item in the currency defined in currency., defaults to None
        :type amount: float, optional
        :param name: The name of the item., defaults to None
        :type name: str, optional
        :param quantity: The quantity of the item., defaults to None
        :type quantity: float, optional
        :param image: The image that appears in the checkout page for this item., defaults to None
        :type image: str, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.name = self._define_str("name", name, nullable=True)
        self.quantity = self._define_number("quantity", quantity, nullable=True)
        self.image = self._define_str("image", image, nullable=True)



from enum import Enum


class GetDisputesListByOrgIdStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACT: "ACT"
    :vartype ACT: str
    :cvar RVW: "RVW"
    :vartype RVW: str
    :cvar PRA: "PRA"
    :vartype PRA: str
    :cvar ARB: "ARB"
    :vartype ARB: str
    :cvar LOS: "LOS"
    :vartype LOS: str
    :cvar WIN: "WIN"
    :vartype WIN: str
    :cvar REV: "REV"
    :vartype REV: str
    """

    ACT = "ACT"
    RVW = "RVW"
    PRA = "PRA"
    ARB = "ARB"
    LOS = "LOS"
    WIN = "WIN"
    REV = "REV"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, GetDisputesListByOrgIdStatus._member_map_.values())
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .coupon import Coupon


@JsonMap({"id_": "id"})
class DiscountCustomerResponse(BaseModel):
    """Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: <BR> 1. Create Coupon, which returns a coupon ID. <BR>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer.

    :param coupon: The coupon can be applied to the following objects - * Subscription - All charges in the subscription are subject to the discount described in the coupon. To add a coupon to a subscription, first use Create Coupon. Then add the coupon ID to the coupon field of the subscription with Create Subscription or Update Subscription. * Customer - The discount applies to all charges to the customer. To add a coupon to a customer, first use Create Coupon. Then add the coupon ID to the coupon field of the customer with Create Customer or Update Customer. * Order - The discount applies to a single order. To add a coupon to an order, first use Create Coupon. The duration field of the coupon must be set to repeating, and the duration_in_months and discount_duration_in_uses fields must be set to 1. Then add the coupon ID to the coupon field of the order with Create an Order or Update Order. A coupon can be applied to one or more customers or subscriptions. However, a subscription can have only one coupon, and the only way to assign two or more coupons to a customer is to assign them to separate subscriptions. If there is a coupon for the customer and another coupon for the customer's subscription, the subscription coupon takes precedence., defaults to None
    :type coupon: Coupon, optional
    :param customer: ID of the Customer object that the discount applies to. String starting with **cus_**. Response only., defaults to None
    :type customer: str, optional
    :param end: The final time that a discount can be used, in Unix time. When the value is -1, the discount never expires. Response only., defaults to None
    :type end: float, optional
    :param id_: ID of the Discount object. String starting with **dis_**. Response only., defaults to None
    :type id_: str, optional
    :param number_of_uses: The number of times that the discount was used for a specific customer or subscription. The initial value is 0. The value is incremented by 1 each time that the discount is used. Response only., defaults to None
    :type number_of_uses: float, optional
    :param start: The time that a discount was created, in Unix time. Response only., defaults to None
    :type start: float, optional
    :param valid: Indicates whether the discount can be used. Response only., defaults to None
    :type valid: bool, optional
    """

    def __init__(
        self,
        coupon: Coupon = None,
        customer: str = None,
        end: float = None,
        id_: str = None,
        number_of_uses: float = None,
        start: float = None,
        valid: bool = None,
    ):
        """Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: <BR> 1. Create Coupon, which returns a coupon ID. <BR>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer.

        :param coupon: The coupon can be applied to the following objects - * Subscription - All charges in the subscription are subject to the discount described in the coupon. To add a coupon to a subscription, first use Create Coupon. Then add the coupon ID to the coupon field of the subscription with Create Subscription or Update Subscription. * Customer - The discount applies to all charges to the customer. To add a coupon to a customer, first use Create Coupon. Then add the coupon ID to the coupon field of the customer with Create Customer or Update Customer. * Order - The discount applies to a single order. To add a coupon to an order, first use Create Coupon. The duration field of the coupon must be set to repeating, and the duration_in_months and discount_duration_in_uses fields must be set to 1. Then add the coupon ID to the coupon field of the order with Create an Order or Update Order. A coupon can be applied to one or more customers or subscriptions. However, a subscription can have only one coupon, and the only way to assign two or more coupons to a customer is to assign them to separate subscriptions. If there is a coupon for the customer and another coupon for the customer's subscription, the subscription coupon takes precedence., defaults to None
        :type coupon: Coupon, optional
        :param customer: ID of the Customer object that the discount applies to. String starting with **cus_**. Response only., defaults to None
        :type customer: str, optional
        :param end: The final time that a discount can be used, in Unix time. When the value is -1, the discount never expires. Response only., defaults to None
        :type end: float, optional
        :param id_: ID of the Discount object. String starting with **dis_**. Response only., defaults to None
        :type id_: str, optional
        :param number_of_uses: The number of times that the discount was used for a specific customer or subscription. The initial value is 0. The value is incremented by 1 each time that the discount is used. Response only., defaults to None
        :type number_of_uses: float, optional
        :param start: The time that a discount was created, in Unix time. Response only., defaults to None
        :type start: float, optional
        :param valid: Indicates whether the discount can be used. Response only., defaults to None
        :type valid: bool, optional
        """
        self.coupon = self._define_object(coupon, Coupon)
        self.customer = self._define_str("customer", customer, nullable=True)
        self.end = self._define_number("end", end, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.number_of_uses = self._define_number(
            "number_of_uses", number_of_uses, nullable=True
        )
        self.start = self._define_number("start", start, nullable=True)
        self.valid = valid



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .beneficiary import Beneficiary
from .payout_ewallets import PayoutEwallets
from .payout_instructions import PayoutInstructions
from .payout_fees import PayoutFees
from .sender import Sender
from .payout_returned_status import PayoutReturnedStatus


class PayoutReturnedPayoutType(Enum):
    """An enumeration representing different categories.

    :cvar CARD: "card"
    :vartype CARD: str
    :cvar CASH: "cash"
    :vartype CASH: str
    :cvar EWALLET: "ewallet"
    :vartype EWALLET: str
    :cvar BANK: "bank"
    :vartype BANK: str
    :cvar RAPYDEWALLET: "rapyd_ewallet"
    :vartype RAPYDEWALLET: str
    """

    CARD = "card"
    CASH = "cash"
    EWALLET = "ewallet"
    BANK = "bank"
    RAPYDEWALLET = "rapyd_ewallet"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, PayoutReturnedPayoutType._member_map_.values())
        )


@JsonMap({"id_": "id"})
class PayoutReturned(BaseModel):
    """PayoutReturned

    :param amount: Amount of the payout, in units defined by payout_currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. Same as payout_amount. Response only., defaults to None
    :type amount: float, optional
    :param beneficiary: beneficiary, defaults to None
    :type beneficiary: Beneficiary, optional
    :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type beneficiary_country: str, optional
    :param created_at: Time of creation of the payout, in Unix time. Response only., defaults to None
    :type created_at: int, optional
    :param description: Description of the payout transaction., defaults to None
    :type description: str, optional
    :param error: Indicates the error code of the last unsuccessful operation on the Payout object., defaults to None
    :type error: str, optional
    :param estimated_time_of_arrival: The estimated time period in which the beneficiary will receive the funds., defaults to None
    :type estimated_time_of_arrival: str, optional
    :param ewallets: An array of one object, which contains the following fields: * ewallet_id - The ID of the wallet that the money is transferred from, a string starting with **ewallet_**. * amount - Amount of the transaction in currency units defined in currency. * percent - A decimal number representing a percentage of the total payout. Response only., defaults to None
    :type ewallets: List[PayoutEwallets], optional
    :param expiration: Determines the day the payout expires, in Unix time. The payout must be completed before the start of this day. Relevant to cash payout methods where the is_expirable field is true in the response to List Payout Method Types., defaults to None
    :type expiration: float, optional
    :param fx_rate: Currency conversion rate for the payout. Decimal. Response only., defaults to None
    :type fx_rate: float, optional
    :param gc_error_code: Reserved. Response only., defaults to None
    :type gc_error_code: str, optional
    :param id_: ID of the payout. String starting with **payout_**., defaults to None
    :type id_: str, optional
    :param identifier_type: Reserved. Response only., defaults to None
    :type identifier_type: str, optional
    :param identifier_value: Reserved. Response only., defaults to None
    :type identifier_value: str, optional
    :param instructions: Describes how the customer collects the payout. Contains the following fields: * name - Short description of the instructions. * steps - A 'steps' object containing a list of steps for the customer to take. Each step is named stepN, where N is an integer. , defaults to None
    :type instructions: List[PayoutInstructions], optional
    :param instructions_value: Additional information from the merchant. For example, the merchant's instructions and transaction number that must be presented for collecting the payout., defaults to None
    :type instructions_value: dict, optional
    :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param paid_amount: Cumulative amount received by the beneficiary, in units defined by payout_currency. Response only., defaults to None
    :type paid_amount: float, optional
    :param paid_at: Time of the payout, in Unix time. Response only., defaults to None
    :type paid_at: str, optional
    :param payout_currency: payout_currency, defaults to None
    :type payout_currency: str, optional
    :param payout_fees: payout_fees, defaults to None
    :type payout_fees: PayoutFees, optional
    :param payout_method_type: The type of payout method. The two-letter prefix must match the beneficiary country computed., defaults to None
    :type payout_method_type: str, optional
    :param payout_type: The type of the payout method., defaults to None
    :type payout_type: PayoutReturnedPayoutType, optional
    :param sender: sender, defaults to None
    :type sender: Sender, optional
    :param sender_amount: Amount that the sender is paying in units of the currency defined in sender_currency.  This amount remains the same regardless of FX fees. Required when payout_amount is not used. Decimal. , defaults to None
    :type sender_amount: float, optional
    :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type sender_country: str, optional
    :param sender_currency: sender_currency, defaults to None
    :type sender_currency: str, optional
    :param statement_descriptor: A statement that includes the reason for the payout. Limited to 35 characters., defaults to None
    :type statement_descriptor: str, optional
    :param status: status, defaults to None
    :type status: PayoutReturnedStatus, optional
    """

    def __init__(
        self,
        amount: float = None,
        beneficiary: Beneficiary = None,
        beneficiary_country: str = None,
        created_at: int = None,
        description: str = None,
        error: str = None,
        estimated_time_of_arrival: str = None,
        ewallets: List[PayoutEwallets] = None,
        expiration: float = None,
        fx_rate: float = None,
        gc_error_code: str = None,
        id_: str = None,
        identifier_type: str = None,
        identifier_value: str = None,
        instructions: List[PayoutInstructions] = None,
        instructions_value: dict = None,
        merchant_reference_id: str = None,
        metadata: dict = None,
        paid_amount: float = None,
        paid_at: str = None,
        payout_currency: str = None,
        payout_fees: PayoutFees = None,
        payout_method_type: str = None,
        payout_type: PayoutReturnedPayoutType = None,
        sender: Sender = None,
        sender_amount: float = None,
        sender_country: str = None,
        sender_currency: str = None,
        statement_descriptor: str = None,
        status: PayoutReturnedStatus = None,
    ):
        """PayoutReturned

        :param amount: Amount of the payout, in units defined by payout_currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. Same as payout_amount. Response only., defaults to None
        :type amount: float, optional
        :param beneficiary: beneficiary, defaults to None
        :type beneficiary: Beneficiary, optional
        :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type beneficiary_country: str, optional
        :param created_at: Time of creation of the payout, in Unix time. Response only., defaults to None
        :type created_at: int, optional
        :param description: Description of the payout transaction., defaults to None
        :type description: str, optional
        :param error: Indicates the error code of the last unsuccessful operation on the Payout object., defaults to None
        :type error: str, optional
        :param estimated_time_of_arrival: The estimated time period in which the beneficiary will receive the funds., defaults to None
        :type estimated_time_of_arrival: str, optional
        :param ewallets: An array of one object, which contains the following fields: * ewallet_id - The ID of the wallet that the money is transferred from, a string starting with **ewallet_**. * amount - Amount of the transaction in currency units defined in currency. * percent - A decimal number representing a percentage of the total payout. Response only., defaults to None
        :type ewallets: List[PayoutEwallets], optional
        :param expiration: Determines the day the payout expires, in Unix time. The payout must be completed before the start of this day. Relevant to cash payout methods where the is_expirable field is true in the response to List Payout Method Types., defaults to None
        :type expiration: float, optional
        :param fx_rate: Currency conversion rate for the payout. Decimal. Response only., defaults to None
        :type fx_rate: float, optional
        :param gc_error_code: Reserved. Response only., defaults to None
        :type gc_error_code: str, optional
        :param id_: ID of the payout. String starting with **payout_**., defaults to None
        :type id_: str, optional
        :param identifier_type: Reserved. Response only., defaults to None
        :type identifier_type: str, optional
        :param identifier_value: Reserved. Response only., defaults to None
        :type identifier_value: str, optional
        :param instructions: Describes how the customer collects the payout. Contains the following fields: * name - Short description of the instructions. * steps - A 'steps' object containing a list of steps for the customer to take. Each step is named stepN, where N is an integer. , defaults to None
        :type instructions: List[PayoutInstructions], optional
        :param instructions_value: Additional information from the merchant. For example, the merchant's instructions and transaction number that must be presented for collecting the payout., defaults to None
        :type instructions_value: dict, optional
        :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param paid_amount: Cumulative amount received by the beneficiary, in units defined by payout_currency. Response only., defaults to None
        :type paid_amount: float, optional
        :param paid_at: Time of the payout, in Unix time. Response only., defaults to None
        :type paid_at: str, optional
        :param payout_currency: payout_currency, defaults to None
        :type payout_currency: str, optional
        :param payout_fees: payout_fees, defaults to None
        :type payout_fees: PayoutFees, optional
        :param payout_method_type: The type of payout method. The two-letter prefix must match the beneficiary country computed., defaults to None
        :type payout_method_type: str, optional
        :param payout_type: The type of the payout method., defaults to None
        :type payout_type: PayoutReturnedPayoutType, optional
        :param sender: sender, defaults to None
        :type sender: Sender, optional
        :param sender_amount: Amount that the sender is paying in units of the currency defined in sender_currency.  This amount remains the same regardless of FX fees. Required when payout_amount is not used. Decimal. , defaults to None
        :type sender_amount: float, optional
        :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type sender_country: str, optional
        :param sender_currency: sender_currency, defaults to None
        :type sender_currency: str, optional
        :param statement_descriptor: A statement that includes the reason for the payout. Limited to 35 characters., defaults to None
        :type statement_descriptor: str, optional
        :param status: status, defaults to None
        :type status: PayoutReturnedStatus, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.beneficiary = self._define_object(beneficiary, Beneficiary)
        self.beneficiary_country = self._define_str(
            "beneficiary_country", beneficiary_country, nullable=True
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.description = self._define_str("description", description, nullable=True)
        self.error = self._define_str("error", error, nullable=True)
        self.estimated_time_of_arrival = self._define_str(
            "estimated_time_of_arrival", estimated_time_of_arrival, nullable=True
        )
        self.ewallets = self._define_list(ewallets, PayoutEwallets)
        self.expiration = self._define_number("expiration", expiration, nullable=True)
        self.fx_rate = self._define_number("fx_rate", fx_rate, nullable=True)
        self.gc_error_code = self._define_str(
            "gc_error_code", gc_error_code, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.identifier_type = self._define_str(
            "identifier_type", identifier_type, nullable=True
        )
        self.identifier_value = self._define_str(
            "identifier_value", identifier_value, nullable=True
        )
        self.instructions = self._define_list(instructions, PayoutInstructions)
        self.instructions_value = instructions_value
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.metadata = metadata
        self.paid_amount = self._define_number(
            "paid_amount", paid_amount, nullable=True
        )
        self.paid_at = self._define_str("paid_at", paid_at, nullable=True)
        self.payout_currency = self._define_str(
            "payout_currency",
            payout_currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.payout_fees = self._define_object(payout_fees, PayoutFees)
        self.payout_method_type = self._define_str(
            "payout_method_type", payout_method_type, nullable=True
        )
        self.payout_type = (
            self._enum_matching(
                payout_type, PayoutReturnedPayoutType.list(), "payout_type"
            )
            if payout_type
            else None
        )
        self.sender = self._define_object(sender, Sender)
        self.sender_amount = self._define_number(
            "sender_amount", sender_amount, nullable=True
        )
        self.sender_country = self._define_str(
            "sender_country", sender_country, nullable=True
        )
        self.sender_currency = self._define_str(
            "sender_currency",
            sender_currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )
        self.status = (
            self._enum_matching(status, PayoutReturnedStatus.list(), "status")
            if status
            else None
        )



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    InlineResponse200_46,
    InlineResponse200_47,
    OrderIdPayBody,
    OrdersOrderIdBody,
    V1OrdersBody,
)


class OrderService(BaseService):

    @cast_models
    def list_order(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        limit: str = None,
        ending_before: str = None,
        starting_after: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_46:
        """Retrieve a list of all orders.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param limit: The maximum number of orders to return. Range - 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param ending_before: The ID of the order created after the last order you want to retrieve., defaults to None
        :type ending_before: str, optional
        :param starting_after: The ID of the order created before the first order you want to retrieve., defaults to None
        :type starting_after: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of orders.
        :rtype: InlineResponse200_46
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/orders", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("limit", limit, explode=False)
            .add_query("ending_before", ending_before, explode=False)
            .add_query("starting_after", starting_after, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_46._unmap(response)

    @cast_models
    def create_order(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: V1OrdersBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_47:
        """Create an order for goods. For services, see Create Subscription.

        :param request_body: The request body., defaults to None
        :type request_body: V1OrdersBody, optional
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The created order.
        :rtype: InlineResponse200_47
        """

        Validator(V1OrdersBody).is_optional().validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/orders", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_47._unmap(response)

    @cast_models
    def retrieve_order(
        self,
        order_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_47:
        """Retrieve the details of an order.

        :param order_id: ID of the order. String starting with **order_**.
        :type order_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: SUCCESS
        :rtype: InlineResponse200_47
        """

        Validator(str).validate(order_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/orders/{{orderId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("orderId", order_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_47._unmap(response)

    @cast_models
    def update_order(
        self,
        request_body: OrdersOrderIdBody,
        order_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_47:
        """Change or modify an order.

        :param request_body: The request body.
        :type request_body: OrdersOrderIdBody
        :param order_id: ID of the order. String starting with **order_**.
        :type order_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: SUCCESS
        :rtype: InlineResponse200_47
        """

        Validator(OrdersOrderIdBody).validate(request_body)
        Validator(str).validate(order_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/orders/{{orderId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("orderId", order_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_47._unmap(response)

    @cast_models
    def pay_order(
        self,
        request_body: OrderIdPayBody,
        order_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_47:
        """Pay an order.

        :param request_body: The request body.
        :type request_body: OrderIdPayBody
        :param order_id: ID of the order. String starting with **order_**.
        :type order_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: SUCCESS
        :rtype: InlineResponse200_47
        """

        Validator(OrderIdPayBody).validate(request_body)
        Validator(str).validate(order_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/orders/{{orderId}}/pay", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("orderId", order_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_47._unmap(response)



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import InlineResponse200_96


class FxService(BaseService):

    @cast_models
    def get_daily_rate(
        self,
        action_type: str,
        buy_currency: str,
        sell_currency: str,
        amount: float = None,
        date_: str = None,
        fixed_side: str = None,
    ) -> InlineResponse200_96:
        """Retrieve a daily rate for conversion of currencies in payments and payouts. Rapyd uses a snapshot of daily foreign exchange rates fetched at 9 PM UTC. The rate returned includes the FX markup fees.

        :param action_type: Determines the type of transaction that the currency exchange applies to. One of the following - payment, payout
        :type action_type: str
        :param buy_currency: Defines the currency purchased in the currency exchange transaction. Three-letter ISO 4217 code in Uppercase.
        :type buy_currency: str
        :param sell_currency: Defines the currency sold in the currency exchange transaction. Three-letter ISO 4217 code in Uppercase.
        :type sell_currency: str
        :param amount: Amount of the currency exchange transaction, in units of the fixed-side currency in Decimal., defaults to None
        :type amount: float, optional
        :param date_: The date when the rate is applicable. Today or earlier. Format YYYY-MM-DD, defaults to None
        :type date_: str, optional
        :param fixed_side: Indicates whether the rate is fixed for the currency defined by buy_currency or sell_currency. One of the following - buy, sell., defaults to None
        :type fixed_side: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieve fixed daily rate
        :rtype: InlineResponse200_96
        """

        Validator(str).validate(action_type)
        Validator(str).validate(buy_currency)
        Validator(str).validate(sell_currency)
        Validator(float).is_optional().validate(amount)
        Validator(str).is_optional().validate(date_)
        Validator(str).is_optional().validate(fixed_side)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/fx_rate", self.get_default_headers())
            .add_query("action_type", action_type)
            .add_query("amount", amount)
            .add_query("buy_currency", buy_currency)
            .add_query("date", date_)
            .add_query("fixed_side", fixed_side)
            .add_query("sell_currency", sell_currency)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_96._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class AccountTransferBody(BaseModel):
    """AccountTransferBody

    :param amount: Amount of the transfer. Decimal.
    :type amount: float
    :param currency: Three-letter ISO 4217 code for the currency used in the amount field. Uppercase.
    :type currency: str
    :param destination_ewallet: ID of the wallet receiving the money. String starting with **ewallet_**.
    :type destination_ewallet: str
    :param expiration: Determines the day the ewallet funds transfer expires, in Unix time. Acceptance of the ewallet transfer must be completed before the start of this day. The default setting is 14 days from the date the ewallet transfer was created., defaults to None
    :type expiration: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param source_ewallet: ID of the wallet sending the money. String starting with **ewallet_**.
    :type source_ewallet: str
    """

    def __init__(
        self,
        amount: float,
        currency: str,
        destination_ewallet: str,
        source_ewallet: str,
        expiration: str = None,
        metadata: dict = None,
    ):
        """AccountTransferBody

        :param amount: Amount of the transfer. Decimal.
        :type amount: float
        :param currency: Three-letter ISO 4217 code for the currency used in the amount field. Uppercase.
        :type currency: str
        :param destination_ewallet: ID of the wallet receiving the money. String starting with **ewallet_**.
        :type destination_ewallet: str
        :param expiration: Determines the day the ewallet funds transfer expires, in Unix time. Acceptance of the ewallet transfer must be completed before the start of this day. The default setting is 14 days from the date the ewallet transfer was created., defaults to None
        :type expiration: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param source_ewallet: ID of the wallet sending the money. String starting with **ewallet_**.
        :type source_ewallet: str
        """
        self.amount = amount
        self.currency = currency
        self.destination_ewallet = destination_ewallet
        self.expiration = self._define_str("expiration", expiration, nullable=True)
        self.metadata = metadata
        self.source_ewallet = source_ewallet



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class SubscriptionPlanUpdatePlanRequest2(BaseModel):
    """SubscriptionPlanUpdatePlanRequest2

    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param nickname: Brief description of the pricing plan., defaults to None
    :type nickname: str, optional
    """

    def __init__(self, metadata: dict = None, nickname: str = None):
        """SubscriptionPlanUpdatePlanRequest2

        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param nickname: Brief description of the pricing plan., defaults to None
        :type nickname: str, optional
        """
        self.metadata = metadata
        self.nickname = self._define_str("nickname", nickname, nullable=True)



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    BankaccountsBankaccounttransfertobankaccountBody,
    BankaccountsVirtualAccountIdBody,
    CardDetailsCardTokenBody,
    CardsActivateBody,
    CardsPersonalizeBody,
    CardsPinBody,
    CardsStatusBody,
    CardTokensGooglePayBody,
    InlineResponse200_107,
    InlineResponse200_108,
    InlineResponse200_109,
    InlineResponse200_110,
    InlineResponse200_74,
    InlineResponse200_75,
    InlineResponse200_76,
    InlineResponse200_77,
    InlineResponse200_78,
    InlineResponse200_79,
    InlineResponse200_80,
    InlineResponse200_81,
    InlineResponse200_82,
    InlineResponse200_83,
    InlineResponse200_84,
    InlineResponse200_93,
    IssuingActivateCardBody,
    IssuingBankaccountsBody,
    IssuingCardsBody,
    IssuingPinBody,
)


class IssuingService(BaseService):

    @cast_models
    def create_v1_hosted_issuing_card_details_by_card_token(
        self,
        request_body: CardDetailsCardTokenBody,
        card_token: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_74:
        """Generate a hosted page that displays details of a virtual issued card directly to the customer. The URL is available for 24 hours after the response is sent, and then it expires.

        :param request_body: The request body.
        :type request_body: CardDetailsCardTokenBody
        :param card_token: ID of the Issued Card Details to Customer.
        :type card_token: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Display Issued Card Details to Customer.
        :rtype: InlineResponse200_74
        """

        Validator(CardDetailsCardTokenBody).validate(request_body)
        Validator(str).validate(card_token)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/hosted/issuing/card_details/{{cardToken}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("cardToken", card_token)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_74._unmap(response)

    @cast_models
    def get_card_issuing_list(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        contact: str = None,
        page_number: float = None,
        page_size: float = None,
        creation_start_date: float = None,
        creation_end_date: float = None,
        activation_start_date: float = None,
        activation_end_date: float = None,
        card_program: str = None,
        status: str = None,
        allow_deleted: bool = None,
        idempotency: str = None,
    ) -> InlineResponse200_75:
        """Retrieve a list of all issuing cards.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param contact: ID of a wallet contact. String starting with **cont_**., defaults to None
        :type contact: str, optional
        :param page_number: Page number to retrieve., defaults to None
        :type page_number: float, optional
        :param page_size: Number of results per page., defaults to None
        :type page_size: float, optional
        :param creation_start_date: Start date of card creation., defaults to None
        :type creation_start_date: float, optional
        :param creation_end_date: End date of card creation., defaults to None
        :type creation_end_date: float, optional
        :param activation_start_date: Start date of card activation., defaults to None
        :type activation_start_date: float, optional
        :param activation_end_date: End date of card activation., defaults to None
        :type activation_end_date: float, optional
        :param card_program: Card program token., defaults to None
        :type card_program: str, optional
        :param status: Card status., defaults to None
        :type status: str, optional
        :param allow_deleted: Is card allow delete., defaults to None
        :type allow_deleted: bool, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of cards.
        :rtype: InlineResponse200_75
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(contact)
        Validator(float).is_optional().validate(page_number)
        Validator(float).is_optional().validate(page_size)
        Validator(float).is_optional().validate(creation_start_date)
        Validator(float).is_optional().validate(creation_end_date)
        Validator(float).is_optional().validate(activation_start_date)
        Validator(float).is_optional().validate(activation_end_date)
        Validator(str).is_optional().validate(card_program)
        Validator(str).is_optional().validate(status)
        Validator(bool).is_optional().validate(allow_deleted)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/issuing/cards", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("contact", contact, explode=False)
            .add_query("page_number", page_number, explode=False)
            .add_query("page_size", page_size, explode=False)
            .add_query("creation_start_date", creation_start_date, explode=False)
            .add_query("creation_end_date", creation_end_date, explode=False)
            .add_query("activation_start_date", activation_start_date, explode=False)
            .add_query("activation_end_date", activation_end_date, explode=False)
            .add_query("card_program", card_program, explode=False)
            .add_query("status", status, explode=False)
            .add_query("allow_deleted", allow_deleted, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_75._unmap(response)

    @cast_models
    def issue_card(
        self,
        request_body: IssuingCardsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_76:
        """Issue a card to a wallet contact.

        :param request_body: The request body.
        :type request_body: IssuingCardsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns card details.
        :rtype: InlineResponse200_76
        """

        Validator(IssuingCardsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/issuing/cards", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_76._unmap(response)

    @cast_models
    def get_card_issuing_details(
        self,
        card_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_77:
        """Retrieve the details of a card.

        :param card_id: ID of a cardId. String starting with **ci_**.
        :type card_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: get Card details By card Id.
        :rtype: InlineResponse200_77
        """

        Validator(str).validate(card_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/{{cardId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("cardId", card_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_77._unmap(response)

    @cast_models
    def activate_card(
        self,
        request_body: CardsActivateBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_78:
        """Activate a card that was issued via the Rapyd issuing platform.

        :param request_body: The request body.
        :type request_body: CardsActivateBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet.
        :rtype: InlineResponse200_78
        """

        Validator(CardsActivateBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/activate", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_78._unmap(response)

    @cast_models
    def hosted_activate_card(
        self,
        request_body: IssuingActivateCardBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_79:
        """A contact can activate his or her physical issued card.<BR>Rapyd sends you webhooks for the following actions:<BR> * Personalize a card - See 'Bulk-Issued Card Personalized Webhook' in online API reference.<BR> * Activate a card - See 'Issued Card Activated Webhook' in online API reference.<BR>**Prerequisites**<BR> * Rapyd Wallet. See 'Rapyd Wallet' in online API reference.<BR> * Wallet contact with a valid phone number. See 'Wallet Contact' in online API reference.

        :param request_body: The request body.
        :type request_body: IssuingActivateCardBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Lists the details of the hosted page.
        :rtype: InlineResponse200_79
        """

        Validator(IssuingActivateCardBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/hosted/issuing/activate_card",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_79._unmap(response)

    @cast_models
    def hosted_card_pin(
        self,
        request_body: IssuingPinBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_80:
        """Create a hosted page for viewing and resetting a PIN.<BR>In the sandbox, you can set any number other than 1111.<BR>Rapyd sends you the 'Issued Card PIN Set' Webhook when the customer sets a Personal Identification Number (PIN).<BR>**Note:**<BR> * PINs are relevant to physical cards.<BR>* Some card programs do not support resetting the PIN. Holders of those cards must reset the PIN at an ATM.<BR>**Prerequisites**<BR>* Rapyd Wallet.<BR>* Wallet contact with a valid phone number.<BR>* Issued card that is activated.

        :param request_body: The request body.
        :type request_body: IssuingPinBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Lists the details of the PIN management hosted page.
        :rtype: InlineResponse200_80
        """

        Validator(IssuingPinBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/hosted/issuing/pin", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_80._unmap(response)

    @cast_models
    def modify_card(
        self,
        request_body: CardsPersonalizeBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_78:
        """Create a connection between an issued card and a wallet contact. Relevant to cards that are issued in bulk and not assigned to any specific person. This method can be used only once per card.

        :param request_body: The request body.
        :type request_body: CardsPersonalizeBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts of a Rapyd Wallet.
        :rtype: InlineResponse200_78
        """

        Validator(CardsPersonalizeBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/personalize",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_78._unmap(response)

    @cast_models
    def update_card_status(
        self,
        request_body: CardsStatusBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_78:
        """Block or unblock a card that was issued via the Rapyd issuing platform.

        :param request_body: The request body.
        :type request_body: CardsStatusBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet,
        :rtype: InlineResponse200_78
        """

        Validator(CardsStatusBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/status", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_78._unmap(response)

    @cast_models
    def get_card_issuing_transactions(
        self,
        card_id: str,
        page_size: str,
        page_number: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        end_date: str = None,
        min_amount: str = None,
        max_amount: str = None,
        merchant_name_search: str = None,
        start_date: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_81:
        """Use the GET method to retrieve a list of transactions for a specific issued card.

        :param card_id: ID of the issued card. String starting with **card_**.
        :type card_id: str
        :param page_size: ID of the transaction, as appears in the array of transactions in the response to 'Retrieve Virtual Account History'.
        :type page_size: str
        :param page_number: Page number to retrieve.
        :type page_number: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param end_date: Timestamp of the last transaction or later, in Unix time., defaults to None
        :type end_date: str, optional
        :param min_amount: Transactions greater than a specific amount., defaults to None
        :type min_amount: str, optional
        :param max_amount: Transactions smaller than a specific amount., defaults to None
        :type max_amount: str, optional
        :param merchant_name_search: Filters the results to return only transactions that have this string as part of the name or location., defaults to None
        :type merchant_name_search: str, optional
        :param start_date: ID of the transaction, as appears in the array of transactions in the response to 'Retrieve Virtual Account History'., defaults to None
        :type start_date: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieve a list of transactions for a specific issued card.
        :rtype: InlineResponse200_81
        """

        Validator(str).validate(card_id)
        Validator(str).validate(page_size)
        Validator(str).validate(page_number)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(end_date)
        Validator(str).is_optional().validate(min_amount)
        Validator(str).is_optional().validate(max_amount)
        Validator(str).is_optional().validate(merchant_name_search)
        Validator(str).is_optional().validate(start_date)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/{{cardId}}/transactions",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("cardId", card_id)
            .add_query("end_date", end_date, explode=False)
            .add_query("min_amount", min_amount, explode=False)
            .add_query("max_amount", max_amount, explode=False)
            .add_query("merchant_name_search", merchant_name_search, explode=False)
            .add_query("page_size", page_size, explode=False)
            .add_query("page_number", page_number, explode=False)
            .add_query("start_date", start_date, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_81._unmap(response)

    @cast_models
    def get_card_issuing_transaction(
        self,
        card_id: str,
        transaction_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_82:
        """Use the GET method to retrieve details of a specific transaction from a specific issued card.

        :param card_id: Card id
        :type card_id: str
        :param transaction_id: Card transaction id
        :type transaction_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns issued card transaction details
        :rtype: InlineResponse200_82
        """

        Validator(str).validate(card_id)
        Validator(str).validate(transaction_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/{{cardId}}/transactions/{{transactionId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("cardId", card_id)
            .add_path("transactionId", transaction_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_82._unmap(response)

    @cast_models
    def set_pin_code(
        self,
        request_body: CardsPinBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_83:
        """Set the PIN code for an issued card.<BR>This method triggers the 'Issued Card PIN Set' Webhook.<BR> **Note:**<BR>* In the sandbox, you can set any number other than 1111.<BR>* Clients with PCI certification can handle personal identifying information for cards, such as card number.<BR> * PINs are relevant to physical cards.

        :param request_body: The request body.
        :type request_body: CardsPinBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns card details.
        :rtype: InlineResponse200_83
        """

        Validator(CardsPinBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/pin", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_83._unmap(response)

    @cast_models
    def add_cardto_google_pay(
        self,
        request_body: CardTokensGooglePayBody,
        card_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_84:
        """Add a card to a Google Pay wallet using Google Pay Push Provisioning.<BR>**Note:**<BR> * PCI certification is required for this method as it contains PCI data.<BR> * The response returns the `getEncryptedPaymentInstrument` string, which is used to add the card to the Google pay wallet by using the Google Pay `pushTokenizeRequest` method. For more information refer to the Google Pay Provisioning developer documentation

        :param request_body: The request body.
        :type request_body: CardTokensGooglePayBody
        :param card_id: ID of the card. String starting with **card_**.
        :type card_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns `PaymentInstrumentData` and `UserAddress`.
        :rtype: InlineResponse200_84
        """

        Validator(CardTokensGooglePayBody).validate(request_body)
        Validator(str).validate(card_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/cards/{{card_id}}/card_tokens/google_pay",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("card_id", card_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_84._unmap(response)

    @cast_models
    def create_issuing(
        self,
        request_body: IssuingBankaccountsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_107:
        """Issue a virtual account number to an existing wallet.

        :param request_body: The request body.
        :type request_body: IssuingBankaccountsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Virtual Account,
        :rtype: InlineResponse200_107
        """

        Validator(IssuingBankaccountsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/bankaccounts", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_107._unmap(response)

    @cast_models
    def simulate_complete_bank_account_issuing_transaction(
        self,
        request_body: BankaccountsBankaccounttransfertobankaccountBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_108:
        """Imulate a deposit to a virtual account number that was issued to a wallet. This method is relevant only for testing in the sandbox. The currency of the transfer must be supported by the specific virtual account. This method triggers the Deposit Completed webhook.

        :param request_body: The request body.
        :type request_body: BankaccountsBankaccounttransfertobankaccountBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet,
        :rtype: InlineResponse200_108
        """

        Validator(BankaccountsBankaccounttransfertobankaccountBody).validate(
            request_body
        )
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/bankaccounts/bankaccounttransfertobankaccount",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_108._unmap(response)

    @cast_models
    def get_remitter_details(
        self,
        virtual_account_id: str,
        transaction_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_109:
        """Retrieve the details of the remitter of a transfer to a virtual bank account.

        :param virtual_account_id: ID of the Virtual Account Number object. String starting with **issuing_**.
        :type virtual_account_id: str
        :param transaction_id: ID of the transaction, as appears in the array of transactions in the response to 'Retrieve Virtual Account History'.
        :type transaction_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet,
        :rtype: InlineResponse200_109
        """

        Validator(str).validate(virtual_account_id)
        Validator(str).validate(transaction_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/bankaccounts/remitters/{{virtualAccountId}}/transactions/{{transaction_id}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("virtualAccountId", virtual_account_id)
            .add_path("transaction_id", transaction_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_109._unmap(response)

    @cast_models
    def retrieve_issuing_by_rapyd_token(
        self,
        virtual_account_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_108:
        """Retrieve a Virtual Account Number object for a wallet.

        :param virtual_account_id: ID of the Virtual Account Number object. String starting with **issuing_**.
        :type virtual_account_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Virtual Account,
        :rtype: InlineResponse200_108
        """

        Validator(str).validate(virtual_account_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/bankaccounts/{{virtualAccountId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("virtualAccountId", virtual_account_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_108._unmap(response)

    @cast_models
    def update_receiving_currency(
        self,
        virtual_account_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: BankaccountsVirtualAccountIdBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_93:
        """Define, change, or cancel the currency to which funds received by a virtual account are converted.<BR>The status of the virtual account must be **ACT** (active) or **PEN** (pending).<BR>To cancel a previously defined currency conversion, set `requested_currency` to **null**.<BR> Prerequisites:<BR>* Create Wallet<BR>* Verify Identity<BR> * Issue Virtual Account to Wallet

        :param request_body: The request body., defaults to None
        :type request_body: BankaccountsVirtualAccountIdBody, optional
        :param virtual_account_id: ID of the Virtual Account Number object. String starting with **issuing_**.
        :type virtual_account_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns the requested currency and bank account data.
        :rtype: InlineResponse200_93
        """

        Validator(BankaccountsVirtualAccountIdBody).is_optional().validate(request_body)
        Validator(str).validate(virtual_account_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/bankaccounts/{{virtualAccountId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("virtualAccountId", virtual_account_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_93._unmap(response)

    @cast_models
    def retrieve_issuing_transaction(
        self,
        virtual_account_id: str,
        transaction_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_110:
        """Retrieve a virtual account transaction.

        :param virtual_account_id: ID of the Virtual Account Number object. String starting with **issuing_**.
        :type virtual_account_id: str
        :param transaction_id: ID of the transaction, as appears in the array of transactions in the response to 'Retrieve Virtual Account History'.
        :type transaction_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet,
        :rtype: InlineResponse200_110
        """

        Validator(str).validate(virtual_account_id)
        Validator(str).validate(transaction_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/issuing/bankaccounts/{{virtualAccountId}}/transactions/{{transactionId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("virtualAccountId", virtual_account_id)
            .add_path("transactionId", transaction_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_110._unmap(response)



from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PayoutInstructions(BaseModel):
    """PayoutInstructions

    :param name: name, defaults to None
    :type name: str, optional
    :param steps: steps, defaults to None
    :type steps: List[dict], optional
    """

    def __init__(self, name: str = None, steps: List[dict] = None):
        """PayoutInstructions

        :param name: name, defaults to None
        :type name: str, optional
        :param steps: steps, defaults to None
        :type steps: List[dict], optional
        """
        self.name = self._define_str("name", name, nullable=True)
        self.steps = steps



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    InlineResponse200_104,
    InlineResponse200_120,
    InlineResponse200_121,
    InlineResponse200_122,
)


class LocalizationMethodsService(BaseService):

    @cast_models
    def get_fx_rate(
        self,
        action_type: str,
        buy_currency: str,
        sell_currency: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        amount: float = None,
        date_: str = None,
        fixed_side: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_104:
        """Retrieve a daily rate for conversion of currencies in payments and payouts. Rapyd uses a snapshot of daily foreign exchange rates fetched at 9 PM UTC. The rate returned includes the FX markup fees.

        :param action_type: Determines the type of transaction that the currency exchange applies to. One of the following - **payment**, **payout**.
        :type action_type: str
        :param buy_currency: Defines the currency purchased in the currency exchange transaction. Three-letter ISO 4217 code in Uppercase.
        :type buy_currency: str
        :param sell_currency: Defines the currency sold in the currency exchange transaction. Three-letter ISO 4217 code in Uppercase.
        :type sell_currency: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param amount: Amount of the currency exchange transaction, in units of the fixed-side currency in Decimal., defaults to None
        :type amount: float, optional
        :param date_: The date when the rate is applicable. Today or earlier. Format **YYYY-MM-DD**, defaults to None
        :type date_: str, optional
        :param fixed_side: Indicates whether the rate is fixed for the currency defined by buy_currency or sell_currency. One of the following - **buy**, **sell**., defaults to None
        :type fixed_side: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieves fixed daily rate
        :rtype: InlineResponse200_104
        """

        Validator(str).validate(action_type)
        Validator(str).validate(buy_currency)
        Validator(str).validate(sell_currency)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(float).is_optional().validate(amount)
        Validator(str).is_optional().validate(date_)
        Validator(str).is_optional().validate(fixed_side)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/fx_rates/", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("action_type", action_type)
            .add_query("amount", amount)
            .add_query("buy_currency", buy_currency)
            .add_query("date", date_)
            .add_query("fixed_side", fixed_side)
            .add_query("sell_currency", sell_currency)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_104._unmap(response)

    @cast_models
    def list_supported_languages(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_120:
        """Retrieve a list of the languages supported for hosted pages.<BR> A hosted page can appear in many foreign languages. The `language` (or `merchant_language`) field determines the default language of a hosted page. The field type is string. Use lower-case letters for the value.<BR> * If no value is specified, the language of the user's browser is used. <BR> * If the language of the user's browser cannot be determined, the default language is English. <BR> Relevant to: <BR>* Display Issued Card Details to Customer <BR> * Create Subscription by Hosted Page <BR> * Create Checkout Page <BR> * Create Payment Link <BR> * Create Beneficiary Tokenization Page <BR> * Create Card Token <BR> * Create Identity Verification Page <BR> * Activate Issued Card Using Hosted Page <BR> * Create Hosted Page for PIN Management

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieves fixed daily rate
        :rtype: InlineResponse200_120
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/hosted/config/supported_languages",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_120._unmap(response)

    @cast_models
    def list_countries(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_121:
        """Retrieve a list of all countries.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of supported countries.
        :rtype: InlineResponse200_121
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/data/countries", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_121._unmap(response)

    @cast_models
    def list_currencies(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_122:
        """Retrieve a list of all currencies.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of currencies.
        :rtype: InlineResponse200_122
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/data/currencies", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_122._unmap(response)



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    EscrowEscrowReleasesBody,
    InlineResponse200_28,
    InlineResponse200_29,
)


class EscrowService(BaseService):

    @cast_models
    def list_escrow_releases(
        self,
        payment: str,
        escrow: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_28:
        """Retrieve a list of all releases of funds from a specified escrow.

        :param payment: ID of the payment. String starting with **payment_**.
        :type payment: str
        :param escrow: ID of the escrow. String starting with **escrow_**.
        :type escrow: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Escrow details after specifying it.
        :rtype: InlineResponse200_28
        """

        Validator(str).validate(payment)
        Validator(str).validate(escrow)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/{{payment}}/escrows/{{escrow}}/escrow_releases",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payment", payment)
            .add_path("escrow", escrow)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_28._unmap(response)

    @cast_models
    def release_funds_from_escrow(
        self,
        payment: str,
        escrow: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: EscrowEscrowReleasesBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_29:
        """Retrieve a list of all releases of funds from a specified escrow.

        :param request_body: The request body., defaults to None
        :type request_body: EscrowEscrowReleasesBody, optional
        :param payment: ID of the payment. String starting with **payment_**.
        :type payment: str
        :param escrow: ID of the escrow. String starting with **escrow_**.
        :type escrow: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Escrow details after creating it.
        :rtype: InlineResponse200_29
        """

        Validator(EscrowEscrowReleasesBody).is_optional().validate(request_body)
        Validator(str).validate(payment)
        Validator(str).validate(escrow)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/{{payment}}/escrows/{{escrow}}/escrow_releases",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payment", payment)
            .add_path("escrow", escrow)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_29._unmap(response)

    @cast_models
    def get_escrow(
        self,
        payment: str,
        escrow: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_28:
        """Retrieve details of the escrow for a payment.

        :param payment: ID of the payment. String starting with **payment_**.
        :type payment: str
        :param escrow: ID of the escrow. String starting with **escrow_**.
        :type escrow: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Escrow details after specifying it.
        :rtype: InlineResponse200_28
        """

        Validator(str).validate(payment)
        Validator(str).validate(escrow)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/{{payment}}/escrows/{{escrow}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("payment", payment)
            .add_path("escrow", escrow)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_28._unmap(response)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .merchant_customer_support import MerchantCustomerSupport
from .hosted_page_status import HostedPageStatus


@JsonMap({"id_": "id"})
class HostedPageActivateCardResponse(BaseModel):
    """HostedPageActivateCardResponse

    :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    :param ewallet_contact: ID of the wallet contact that the card is assigned to. Must have a valid phone number. String starting with **cont_**., defaults to None
    :type ewallet_contact: str, optional
    :param geo_country: Reserved., defaults to None
    :type geo_country: str, optional
    :param id_: ID of the hosted page for activating a card, a string starting with **hp_issuing_act_**., defaults to None
    :type id_: str, optional
    :param language: Determines the default language of the hosted page.\<BR\>* When this parameter is null, the language of the user's browser is used.\<BR\>* If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param merchant_alias: Client's name., defaults to None
    :type merchant_alias: str, optional
    :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal., defaults to None
    :type merchant_color: str, optional
    :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
    :type merchant_customer_support: MerchantCustomerSupport, optional
    :param merchant_logo: URL for the image of the client's logo. To configure this field, use the Client Portal., defaults to None
    :type merchant_logo: str, optional
    :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers.\<BR\>To configure this field, use the Client Portal., defaults to None
    :type merchant_privacy_policy: str, optional
    :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Porta, defaults to None
    :type merchant_terms: str, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when `cancel_url`, `complete_url` or both fields are not set. To configure this field, use the Client Portal., defaults to None
    :type merchant_website: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation.\<BR\>**Range**: 1 minute to 30 days., defaults to None
    :type page_expiration: float, optional
    :param personalize: When **true**, connects an issued card to a wallet contact. Relevant to a card that was issued in bulk and is not assigned to a specific person. Transparent to the customer. See also 'Personalize Bulk-Issued Card' in online documentation., defaults to None
    :type personalize: float, optional
    :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
    :type redirect_url: str, optional
    :param region: Reserved., defaults to None
    :type region: str, optional
    :param skip_pin: When **true**, the customer cannot reset the PIN via a hosted page., defaults to None
    :type skip_pin: bool, optional
    :param status: Status of the hosted page. One of the following: NEW - The hosted page was created. DON - Done. The card was added to the customer profile. EXP - The hosted page expired. , defaults to None
    :type status: HostedPageStatus, optional
    """

    def __init__(
        self,
        cancel_url: str = None,
        complete_url: str = None,
        ewallet_contact: str = None,
        geo_country: str = None,
        id_: str = None,
        language: str = None,
        merchant_alias: str = None,
        merchant_color: str = None,
        merchant_customer_support: MerchantCustomerSupport = None,
        merchant_logo: str = None,
        merchant_privacy_policy: str = None,
        merchant_terms: str = None,
        merchant_website: str = None,
        page_expiration: float = None,
        personalize: float = None,
        redirect_url: str = None,
        region: str = None,
        skip_pin: bool = None,
        status: HostedPageStatus = None,
    ):
        """HostedPageActivateCardResponse

        :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        :param ewallet_contact: ID of the wallet contact that the card is assigned to. Must have a valid phone number. String starting with **cont_**., defaults to None
        :type ewallet_contact: str, optional
        :param geo_country: Reserved., defaults to None
        :type geo_country: str, optional
        :param id_: ID of the hosted page for activating a card, a string starting with **hp_issuing_act_**., defaults to None
        :type id_: str, optional
        :param language: Determines the default language of the hosted page.\<BR\>* When this parameter is null, the language of the user's browser is used.\<BR\>* If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param merchant_alias: Client's name., defaults to None
        :type merchant_alias: str, optional
        :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal., defaults to None
        :type merchant_color: str, optional
        :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
        :type merchant_customer_support: MerchantCustomerSupport, optional
        :param merchant_logo: URL for the image of the client's logo. To configure this field, use the Client Portal., defaults to None
        :type merchant_logo: str, optional
        :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers.\<BR\>To configure this field, use the Client Portal., defaults to None
        :type merchant_privacy_policy: str, optional
        :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Porta, defaults to None
        :type merchant_terms: str, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when `cancel_url`, `complete_url` or both fields are not set. To configure this field, use the Client Portal., defaults to None
        :type merchant_website: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation.\<BR\>**Range**: 1 minute to 30 days., defaults to None
        :type page_expiration: float, optional
        :param personalize: When **true**, connects an issued card to a wallet contact. Relevant to a card that was issued in bulk and is not assigned to a specific person. Transparent to the customer. See also 'Personalize Bulk-Issued Card' in online documentation., defaults to None
        :type personalize: float, optional
        :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
        :type redirect_url: str, optional
        :param region: Reserved., defaults to None
        :type region: str, optional
        :param skip_pin: When **true**, the customer cannot reset the PIN via a hosted page., defaults to None
        :type skip_pin: bool, optional
        :param status: Status of the hosted page. One of the following: NEW - The hosted page was created. DON - Done. The card was added to the customer profile. EXP - The hosted page expired. , defaults to None
        :type status: HostedPageStatus, optional
        """
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.ewallet_contact = self._define_str(
            "ewallet_contact", ewallet_contact, nullable=True
        )
        self.geo_country = self._define_str("geo_country", geo_country, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.language = self._define_str("language", language, nullable=True)
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_customer_support = self._define_object(
            merchant_customer_support, MerchantCustomerSupport
        )
        self.merchant_logo = self._define_str(
            "merchant_logo", merchant_logo, nullable=True
        )
        self.merchant_privacy_policy = self._define_str(
            "merchant_privacy_policy", merchant_privacy_policy, nullable=True
        )
        self.merchant_terms = self._define_str(
            "merchant_terms", merchant_terms, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.personalize = self._define_number(
            "personalize", personalize, nullable=True
        )
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.region = self._define_str("region", region, nullable=True)
        self.skip_pin = skip_pin
        self.status = (
            self._enum_matching(status, HostedPageStatus.list(), "status")
            if status
            else None
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_103_data_bank_accounts import (
    InlineResponse200_103DataBankAccounts,
)


@JsonMap({})
class InlineResponse200_103Data(BaseModel):
    """InlineResponse200_103Data

    :param bank_accounts: Array of objects with details of the virtual accounts. Each object contains the following fields, defaults to None
    :type bank_accounts: InlineResponse200_103DataBankAccounts, optional
    """

    def __init__(self, bank_accounts: InlineResponse200_103DataBankAccounts = None):
        """InlineResponse200_103Data

        :param bank_accounts: Array of objects with details of the virtual accounts. Each object contains the following fields, defaults to None
        :type bank_accounts: InlineResponse200_103DataBankAccounts, optional
        """
        self.bank_accounts = self._define_object(
            bank_accounts, InlineResponse200_103DataBankAccounts
        )



"""
An enum class containing all the possible environments for the SDK
"""

from enum import Enum


class Environment(Enum):
    """The environments available for the SDK"""

    DEFAULT = "https://sandboxapi.rapyd.net"
    SANDBOX = "https://sandboxapi.rapyd.net"
    PRODUCTION = "https://api.rapyd.net"



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class V1ordersItemsType(Enum):
    """An enumeration representing different categories.

    :cvar SHIPPING: "shipping"
    :vartype SHIPPING: str
    :cvar SKU: "sku"
    :vartype SKU: str
    """

    SHIPPING = "shipping"
    SKU = "sku"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, V1ordersItemsType._member_map_.values()))


@JsonMap({"type_": "type"})
class V1ordersItems(BaseModel):
    """V1ordersItems

    :param amount: Price of one SKU unit, in the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
    :type amount: str, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param description: Description of the item., defaults to None
    :type description: str, optional
    :param parent: ID of the SKU object that represents the product. String starting with sku_. Each SKU can appear in this list only one time, for items of type 'sku'. Required when type is sku. Must be null when type is shipping., defaults to None
    :type parent: str, optional
    :param type_: Type of line item., defaults to None
    :type type_: V1ordersItemsType, optional
    :param quantity: Quantity of the product in the line item. Integer. Required when type is sku., defaults to None
    :type quantity: float, optional
    """

    def __init__(
        self,
        amount: str = None,
        currency: str = None,
        description: str = None,
        parent: str = None,
        type_: V1ordersItemsType = None,
        quantity: float = None,
    ):
        """V1ordersItems

        :param amount: Price of one SKU unit, in the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
        :type amount: str, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param description: Description of the item., defaults to None
        :type description: str, optional
        :param parent: ID of the SKU object that represents the product. String starting with sku_. Each SKU can appear in this list only one time, for items of type 'sku'. Required when type is sku. Must be null when type is shipping., defaults to None
        :type parent: str, optional
        :param type_: Type of line item., defaults to None
        :type type_: V1ordersItemsType, optional
        :param quantity: Quantity of the product in the line item. Integer. Required when type is sku., defaults to None
        :type quantity: float, optional
        """
        self.amount = self._define_str("amount", amount, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.description = self._define_str("description", description, nullable=True)
        self.parent = self._define_str("parent", parent, nullable=True)
        self.type_ = (
            self._enum_matching(type_, V1ordersItemsType.list(), "type_")
            if type_
            else None
        )
        self.quantity = self._define_number("quantity", quantity, nullable=True)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .status_1 import Status1
from .customer import Customer


@JsonMap({})
class InlineResponse200_37(BaseModel):
    """InlineResponse200_37

    :param status: status, defaults to None
    :type status: Status1, optional
    :param data: customer, defaults to None
    :type data: Customer, optional
    """

    def __init__(self, status: Status1 = None, data: Customer = None):
        """InlineResponse200_37

        :param status: status, defaults to None
        :type status: Status1, optional
        :param data: customer, defaults to None
        :type data: Customer, optional
        """
        self.status = self._define_object(status, Status1)
        self.data = self._define_object(data, Customer)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class V1InvoiceItemsBody(BaseModel):
    """V1InvoiceItemsBody

    :param amount: The amount of the total charge or credit for this item. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015.\<BR\> This is `quantity` times `unit_amount`. A credit is indicated by a negative number., defaults to None
    :type amount: float, optional
    :param currency: Three-letter ISO 4217 code for the currency used in the `amount` field.
    :type currency: str
    :param customer: ID of the customer. String starting with **cus_**.
    :type customer: str
    :param description: Description of the invoice item., defaults to None
    :type description: str, optional
    :param discountable: Determines whether this invoice item is subject to the discount defined in the coupon that is assigned to the customer or subscription.\<BR\> For negative amounts and prorations, the default is **false**, and for all other invoice items, the default is **true**., defaults to None
    :type discountable: bool, optional
    :param invoice: ID of the invoice that this invoice item is assigned to. Relevant when `subscription` is not set., defaults to None
    :type invoice: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param quantity: Indicates the number of units charged as a single invoice item. Integer., defaults to None
    :type quantity: float, optional
    :param subscription: ID of the subscription to assign this invoice item to. By default, the invoice item is assigned to the customer's subscription whose current billing cycle ends first. Relevant when `invoice` is not set., defaults to None
    :type subscription: str, optional
    :param unit_amount: Per-unit price of the product or service, adjusted as defined in the plan. Decimal., defaults to None
    :type unit_amount: float, optional
    """

    def __init__(
        self,
        currency: str,
        customer: str,
        amount: float = None,
        description: str = None,
        discountable: bool = None,
        invoice: str = None,
        metadata: dict = None,
        quantity: float = None,
        subscription: str = None,
        unit_amount: float = None,
    ):
        """V1InvoiceItemsBody

        :param amount: The amount of the total charge or credit for this item. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015.\<BR\> This is `quantity` times `unit_amount`. A credit is indicated by a negative number., defaults to None
        :type amount: float, optional
        :param currency: Three-letter ISO 4217 code for the currency used in the `amount` field.
        :type currency: str
        :param customer: ID of the customer. String starting with **cus_**.
        :type customer: str
        :param description: Description of the invoice item., defaults to None
        :type description: str, optional
        :param discountable: Determines whether this invoice item is subject to the discount defined in the coupon that is assigned to the customer or subscription.\<BR\> For negative amounts and prorations, the default is **false**, and for all other invoice items, the default is **true**., defaults to None
        :type discountable: bool, optional
        :param invoice: ID of the invoice that this invoice item is assigned to. Relevant when `subscription` is not set., defaults to None
        :type invoice: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param quantity: Indicates the number of units charged as a single invoice item. Integer., defaults to None
        :type quantity: float, optional
        :param subscription: ID of the subscription to assign this invoice item to. By default, the invoice item is assigned to the customer's subscription whose current billing cycle ends first. Relevant when `invoice` is not set., defaults to None
        :type subscription: str, optional
        :param unit_amount: Per-unit price of the product or service, adjusted as defined in the plan. Decimal., defaults to None
        :type unit_amount: float, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.currency = currency
        self.customer = customer
        self.description = self._define_str("description", description, nullable=True)
        self.discountable = discountable
        self.invoice = self._define_str("invoice", invoice, nullable=True)
        self.metadata = metadata
        self.quantity = self._define_number("quantity", quantity, nullable=True)
        self.subscription = self._define_str(
            "subscription", subscription, nullable=True
        )
        self.unit_amount = self._define_number(
            "unit_amount", unit_amount, nullable=True
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .status_1 import Status1
from .coupon import Coupon


@JsonMap({})
class InlineResponse200_51(BaseModel):
    """InlineResponse200_51

    :param status: status, defaults to None
    :type status: Status1, optional
    :param data: The coupon can be applied to the following objects - * Subscription - All charges in the subscription are subject to the discount described in the coupon. To add a coupon to a subscription, first use Create Coupon. Then add the coupon ID to the coupon field of the subscription with Create Subscription or Update Subscription. * Customer - The discount applies to all charges to the customer. To add a coupon to a customer, first use Create Coupon. Then add the coupon ID to the coupon field of the customer with Create Customer or Update Customer. * Order - The discount applies to a single order. To add a coupon to an order, first use Create Coupon. The duration field of the coupon must be set to repeating, and the duration_in_months and discount_duration_in_uses fields must be set to 1. Then add the coupon ID to the coupon field of the order with Create an Order or Update Order. A coupon can be applied to one or more customers or subscriptions. However, a subscription can have only one coupon, and the only way to assign two or more coupons to a customer is to assign them to separate subscriptions. If there is a coupon for the customer and another coupon for the customer's subscription, the subscription coupon takes precedence., defaults to None
    :type data: Coupon, optional
    """

    def __init__(self, status: Status1 = None, data: Coupon = None):
        """InlineResponse200_51

        :param status: status, defaults to None
        :type status: Status1, optional
        :param data: The coupon can be applied to the following objects - * Subscription - All charges in the subscription are subject to the discount described in the coupon. To add a coupon to a subscription, first use Create Coupon. Then add the coupon ID to the coupon field of the subscription with Create Subscription or Update Subscription. * Customer - The discount applies to all charges to the customer. To add a coupon to a customer, first use Create Coupon. Then add the coupon ID to the coupon field of the customer with Create Customer or Update Customer. * Order - The discount applies to a single order. To add a coupon to an order, first use Create Coupon. The duration field of the coupon must be set to repeating, and the duration_in_months and discount_duration_in_uses fields must be set to 1. Then add the coupon ID to the coupon field of the order with Create an Order or Update Order. A coupon can be applied to one or more customers or subscriptions. However, a subscription can have only one coupon, and the only way to assign two or more coupons to a customer is to assign them to separate subscriptions. If there is a coupon for the customer and another coupon for the customer's subscription, the subscription coupon takes precedence., defaults to None
        :type data: Coupon, optional
        """
        self.status = self._define_object(status, Status1)
        self.data = self._define_object(data, Coupon)



from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class V1RefundsBody(BaseModel):
    """V1RefundsBody

    :param amount: The amount of the refund. Decimal., defaults to None
    :type amount: float, optional
    :param currency: The currency of the amount received by the original payment source. Three-letter ISO 4217 code., defaults to None
    :type currency: str, optional
    :param ewallets: ID of the wallet contact that the card is assigned to. String starting with **cont_**., defaults to None
    :type ewallets: List[str], optional
    :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payment: ID of the Payment object that the refund is charged against. String starting with **payment_**.
    :type payment: str
    :param reason: Description of the reason for the refund., defaults to None
    :type reason: str, optional
    """

    def __init__(
        self,
        payment: str,
        amount: float = None,
        currency: str = None,
        ewallets: List[str] = None,
        merchant_reference_id: str = None,
        metadata: dict = None,
        reason: str = None,
    ):
        """V1RefundsBody

        :param amount: The amount of the refund. Decimal., defaults to None
        :type amount: float, optional
        :param currency: The currency of the amount received by the original payment source. Three-letter ISO 4217 code., defaults to None
        :type currency: str, optional
        :param ewallets: ID of the wallet contact that the card is assigned to. String starting with **cont_**., defaults to None
        :type ewallets: List[str], optional
        :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payment: ID of the Payment object that the refund is charged against. String starting with **payment_**.
        :type payment: str
        :param reason: Description of the reason for the refund., defaults to None
        :type reason: str, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.ewallets = ewallets
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.metadata = metadata
        self.payment = payment
        self.reason = self._define_str("reason", reason, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class OrderIdPayBody(BaseModel):
    """OrderIdPayBody

    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payment_method: ID of the payment method. String starting with card_ or other_. If not specified, the payment method is the default_payment_method of the customer., defaults to None
    :type payment_method: str, optional
    :param customer: ID of a customer. String starting with cus_. The order is paid with the customer's default payment method., defaults to None
    :type customer: str, optional
    """

    def __init__(
        self, metadata: dict = None, payment_method: str = None, customer: str = None
    ):
        """OrderIdPayBody

        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payment_method: ID of the payment method. String starting with card_ or other_. If not specified, the payment method is the default_payment_method of the customer., defaults to None
        :type payment_method: str, optional
        :param customer: ID of a customer. String starting with cus_. The order is paid with the customer's default payment method., defaults to None
        :type customer: str, optional
        """
        self.metadata = metadata
        self.payment_method = self._define_str(
            "payment_method", payment_method, nullable=True
        )
        self.customer = self._define_str("customer", customer, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class SkuPackageDimensions(BaseModel):
    """Physical attributes of the SKU item. Contains the following fields, height length weight width These fields are represented as numbers, but it is the responsibility of the merchant to define and interpret the relevant units of length and weight.

    :param height: NA, defaults to None
    :type height: float, optional
    :param length: NA, defaults to None
    :type length: float, optional
    :param weight: NA, defaults to None
    :type weight: float, optional
    :param width: NA, defaults to None
    :type width: float, optional
    """

    def __init__(
        self,
        height: float = None,
        length: float = None,
        weight: float = None,
        width: float = None,
    ):
        """Physical attributes of the SKU item. Contains the following fields, height length weight width These fields are represented as numbers, but it is the responsibility of the merchant to define and interpret the relevant units of length and weight.

        :param height: NA, defaults to None
        :type height: float, optional
        :param length: NA, defaults to None
        :type length: float, optional
        :param weight: NA, defaults to None
        :type weight: float, optional
        :param width: NA, defaults to None
        :type width: float, optional
        """
        self.height = self._define_number("height", height, nullable=True)
        self.length = self._define_number("length", length, nullable=True)
        self.weight = self._define_number("weight", weight, nullable=True)
        self.width = self._define_number("width", width, nullable=True)
